;--- DESS IRR03 --- Le 05/01/2003 ---
; Alexandre ALAPETITE et Pierre COHADE
;
; Nom du fichier:/users/g1/irr/irr03/prive/lisp/csp/
;    "INTERFACE_HM.lisp".
;
; Rle du fichier:
;    Ce fichier contient les fonctions qui font l'interface entre l'homme et la machine.
;    Il transforme les lignes de commandes de l'utilisateur en lignes de commandes appropries pour le noyau
;    et il les stockes dans la PLISTE "VARIABLES".
;

;Dfinition des diffrents oprateurs
(DEFVAR OPERATEURS '(= < > <= >= /= + - * /))

;Fonction qui sert  afficher la PListe
(DEFUN AFFICHE_SOLUTION(LX liste_variables)
  (IF (NULL liste_variables)
    T
    (PROGN
      (format t "  ~a = ~a ~%"(FIRST liste_variables)(GET_VALEUR (GET_VARIABLE LX (FIRST liste_variables))))
      (AFFICHE_SOLUTION LX (REST liste_variables))
    )
  )
)

;retourne si la variable est un oprateur
(DEFUN IS_OPERATOR(X)
  (NOT (NULL (MEMBER X OPERATEURS)))
)

;passe une expression compltement parenthse de infix  prfix
(DEFUN PREFIXER(L)
  (IF (LISTP L)
    (LIST (CADR L) (PREFIXER (CAR L)) (PREFIXER (CADDR L)))
    L
  )
)

;ajoute toutes les parenthses  une expression infixe
(DEFUN PARENTHESER(L)
  (IF (LISTP L)
    (iF (NULL (CADDDR L))                     ;si on est dans un triplet, on explore les sous-listes
      (LIST
        (PARENTHESER (CAR L))                 ;terme 1
        (CADR L)                              ;forcement oprateur
        (PARENTHESER (CADDR L))               ;terme 2
      )
      (PARENTHESER (NCONC                     ;sinon on divise la liste en un triplet, puis le reste. rcursive.
        (LIST
          (LIST (CAR L) (CADR L) (CADDR L))   ;le triplet terme 1, oprateur, terme 2
          (CADDDR L)                          ;forcement un oprateur
        )
        (CDDDDR L)                            ;le reste de la liste
      ))
    )
    L
  )
)

;transforme une liste classique en une expression valuable prfixe
(DEFUN TRANSFORM_EVALUABLE(L)
  (IF (NULL L)
    NIL
    (IF (IS_OPERATOR (FIRST L))
     (PARENTHESER L)             ;l'expression est dj prfixe
     (PREFIXER (PARENTHESER L))  ;il faut prfixer l'expression
    )
  )
)

;ajoute une contrainte prfixe
(DEFUN AJOUT_CONTRAINTE_AUTO(contrainte)
  (AJOUT_CONTRAINTE VARIABLES (TRANSFORM_EVALUABLE contrainte))
)

;permet de renvoyer l'intervalle dsir
(DEFUN CALCUL_INTERVALLE(debut_intervalle fin_intervalle)
  (IF (EQUAL debut_intervalle fin_intervalle)
    (LIST fin_intervalle)
    (NCONC (LIST debut_intervalle) (CALCUL_INTERVALLE (+ 1 debut_intervalle) fin_intervalle))
  )
)

;permet de selectionner si la donne entre est une contrainte
;ou une variable avec son domaine et la memorise
(DEFUN SELECT_AND_MEMORISE(donnees)
  (IF (OR (OR (IS_OPERATOR (FIRST donnees)) (LISTP (FIRST donnees))) (NUMBERP (FIRST donnees)))
    (AJOUT_CONTRAINTE_AUTO donnees)
    (IF (IS_OPERATOR (CADR donnees))
      (AJOUT_CONTRAINTE_AUTO donnees)
      (IF (LISTP (CADR donnees))
         (SET_DOMAINE VARIABLES (FIRST donnees) (CADR donnees))
         (SET_DOMAINE VARIABLES (FIRST donnees) (CALCUL_INTERVALLE (CADR donnees) (CADDR donnees)))
      )
    )
  )
)

;permet de selectionner si la donne entre est une contrainte
;ou une variable avec son domaine et la memorise
(DEFUN LISTE_SELECT_AND_MEMORISE(liste_donnees)
  (IF (NULL liste_donnees)
    NIL
    (PROGN
      (SELECT_AND_MEMORISE (FIRST liste_donnees))
      (LISTE_SELECT_AND_MEMORISE (REST liste_donnees))
    )
  )
)

; Lit les fichiers, selectionne le format de celui-ci et stocke ces donnes dans la PLISTE
(DEFUN LECTURE_FICHIER()
  (with-open-file (ifile (CONCATENATE 'STRING "./PROBLEMES/" (STRING (READ T))) :IF-DOES-NOT-EXIST NIL)
    (IF (NULL ifile)
      (PROGN
        (format t "~%")
        (format t "----------------------------------------~2%")
        (format t "Le fichier n'existe pas !!!~%")
      )
      (LET ((LEC_DONNEES NIL))
        (RAZ VARIABLES) ;On efface la PLISTE
        (SETF LEC_DONNEES (READ ifile))
        (IF (AND (LISTP (FIRST LEC_DONNEES)) (LISTP (CADR LEC_DONNEES)))
          (PROGN ;fichier au format: Liste des variables, Liste des Domaines et Liste des contraintes
            (LISTE_SELECT_AND_MEMORISE LEC_DONNEES)
            (SETF LEC_DONNEES (READ ifile))
            (LISTE_SELECT_AND_MEMORISE LEC_DONNEES)
            (MENU_AFFICHAGE_PLISTE)
          )
          (PROGN ;fichier au format des donnes entres au clavier
            (format t "~2%Donnes rcupres:~%")
            (do ((FIN_FICHIER (NULL LEC_DONNEES)))
                ((EQUAL FIN_FICHIER T))
              (PROGN
                (SELECT_AND_MEMORISE LEC_DONNEES)
                (format t "     -> ~a ~%" LEC_DONNEES)
                (SETF LEC_DONNEES (READ ifile))
                (SETF FIN_FICHIER (NULL LEC_DONNEES))
              )
            )
          )
        )
      )
    )
  )
)

;affiche le menu principal
(DEFUN AFFICHE_MENU_PRINCIPAL()
  (PROGN
    (format t "~%")
    (format t "----------------------------------------~%")
    (format t "MENU PRINCIPAL:~%")
    (format t "  1: Menu des donnes.~%")
    (format t "  2: Lancer la rsolution par BackTrack.~%")
    (format t "  3: Lancer la rsolution par AC3 Modifi.~%")
    (format t "  4: Lancer la rsolution par FC.~%")
    (format t "  5: Quitter le programme.~%")
    (format t "~%")
    (format t "Selection:~%")
  )
)

;menu donnes
(DEFUN MENU_DONNEES()
  (PROGN
    (do ((LEC_DON 0))
        ((EQUAL LEC_DON 1))
      (PROGN
        (format t "~%")
        (format t "----------------------------------------~%")
        (format t "MENU DONNEES:~%")
        (format t "  1: Charger les donnes d'un fichier.~%")
        (format t "  2: Entrer des donnes ( partir du clavier).~%")
        (format t "  3: Afficher les donnes ('PListe').~%")
        (format t "  4: Retour au Menu Principal.~%")
        (format t "~%")
        (format t "Choix:~%")
        (CASE (READ T)
          (1 (MENU_FICHIER))
          (2 (MENU_ENTRER_DONNEES))
          (3 (MENU_AFFICHAGE_PLISTE))
          (4 (IF (OR (NULL (SYMBOL-PLIST VARIABLES)) (NULL (LISTE_VARIABLES_NO_DOMAINE VARIABLES)))
               (SETF LEC_DON 1)
               (PROGN
                 (format t " ~%")
                 (format t "---------------------------------------- ~%")
                 (format t "Il n'est pas possible de quitter le Menu des donnes car: ~%")
                 (do ((FIN_VAR (LISTE_VARIABLES_NO_DOMAINE VARIABLES)))
                     ((EQUAL FIN_VAR NIL))
                   (PROGN
                     (format t "  ->Le domaine de la Variable ~a n'est pas dfinit !!! ~%" (FIRST FIN_VAR))
                     (SETF FIN_VAR (REST FIN_VAR))
                   )
                 )
               )
             ))
        )
      )
    )
    (CREER_LISTE_CHECK_FORWARD VARIABLES)
  )
)

;lecture du fichier
(DEFUN MENU_FICHIER()
  (PROGN
    (format t "~%")
    (format t "----------------------------------------~%")
    (format t "Nom du fichier  charger:~%")
    (LECTURE_FICHIER)
  )
)

;lecture des donnes  partir du clavier
(DEFUN MENU_ENTRER_DONNEES()
  (PROGN
    (format t "~%")
    (format t "----------------------------------------~%")
    (format t "Entrer la contrainte, la variable ou le domaine:~%")
    (SELECT_AND_MEMORISE (READ T))
  )
)

;Affiche la PListe
(DEFUN MENU_AFFICHAGE_PLISTE()
  (IF (NULL (FIRST (LISTE_VARIABLES VARIABLES)))
    (PROGN
      (format t "~%")
      (format t "----------------------------------------~2%")
      (format t "La PListe 'VARIABLES' est vide !~%")
    )
    (PROGN
      (format t "~%")
      (format t "----------------------------------------~%")
      (format t "AFFIGAGE DE LA PLISTE 'VARIABLES' :~%")
      (format t "~%")
      (loop for var in (LISTE_VARIABLES VARIABLES) do
        (format t "   -> ~a: ~a~%" var (GET_VARIABLE VARIABLES var))
      )
    )
  )
)

;lancement du CSP
(DEFUN LANCE_BT(LX)
  (PROGN
    (format t "~%")
    (format t "----------------------------------------~%")
    (format t "Solution trouve par CSP:~2%")
    (IF (EQUAL (PROG_BT LX) T)
      (AFFICHE_SOLUTION LX (LISTE_VARIABLES LX))
      (format t " Aucune solution trouve !~%")
    )
  )
)

;lancement du AC3
(DEFUN LANCE_AC3(LX)
  (PROGN
    (format t "~%")
    (format t "----------------------------------------~%")
    (IF (EQUAL (PROG_FILTRE_PARTIAL_LOOK_FUTURE LX) T)
      (PROGN
        (format t "~%--Fin du filtre AC3--~2%")
        (format t "Solution trouve par AC3:~2%")
        (IF (EQUAL (PROG_BT LX) T)
          (AFFICHE_SOLUTION LX (LISTE_VARIABLES LX))
          (format t " Aucune solution trouve !~%")
        )
      )
      (format t "~% Aucune solution possible aprs filtrage~2%")
    )
  )
)

;lancement du FC
(DEFUN LANCE_FC(LX)
  (PROGN
    (format t "~%")
    (format t "----------------------------------------~%")
    (format t "Solution trouve par FC:~2%")
    (IF (EQUAL (PROG_FC LX) T)
      (AFFICHE_SOLUTION LX (LISTE_VARIABLES LX))
      (format t " Aucune solution trouve !~%")
    )
  )
)
